@@ -1,4 +1,4 @@
 from rdfobject import RDFobject, URINotSetException, N3NotUnderstoodException, TextInputSource
 from urihelper import URIHelper, NotANamespaceException, PrefixNotKnownException, NAMESPACES
-from fileentity import FileEntityFactory
+from fileentity import FileEntityFactory, FileMultiEntityFactory
 from fedoraobjentity import FedoraEntityFactory
