@@ -11,6 +11,8 @@ from rdfobject.stores import ItemDoesntExistException
 
 from rdflib import ConjunctiveGraph
 
+from collections import defaultdict
+
 import simplejson
 
 import logging
@@ -18,17 +20,18 @@ import logging
 def _init_obj_wrapper(fn):
     def new(self, *args, **kw):
         if not self.obj:
+            self.context = defaultdict(list)
             self.obj = self.s.getObject(self.id)
             self.uri = self.obj.uri
         return fn(self, *args, **kw)
     return new
 
 class StoredEntity(Entity):
-    def init(self, id=None, storageclient=None, **context):
+    def init(self, id=None, storageclient=None, **logcontext):
         self.id = id
         self.s = storageclient
         self.obj = None
-        self.context = context
+        self.logcontext = logcontext
         self.revert()
 
     @_init_obj_wrapper
@@ -36,6 +39,39 @@ class StoredEntity(Entity):
         self.store_root()
         self.store_manifest()
         self.store_parts()
+        self.s.log_audit(self.id, self.logcontext, self.context)
+        # Change delta logged...
+        self.context = defaultdict(list)
+
+    @_init_obj_wrapper
+    def add_namespace(self, prefix, ns):
+        super(StoredEntity, self).add_namespace(prefix, ns)
+        self.context['addnamespace'].append((prefix, ns))
+
+    @_init_obj_wrapper
+    def del_namespace(self, prefix):
+        super(StoredEntity, self).del_namespace(prefix)
+        self.context['delnamespace'].append((prefix, ns))
+        
+    @_init_obj_wrapper
+    def add_triple(self, s,p,o, create_item=True):
+        super(StoredEntity, self).add_triple(s,p,o, create_item=create_item)
+        self.context['add'].append((s,p,o.__str__()))
+        
+    @_init_obj_wrapper
+    def del_triple(self, s,p,o=None):
+        super(StoredEntity, self).del_triple(s,p,o)
+        self.context['del'].append((s,p,o.__str__()))
+        
+    @_init_obj_wrapper
+    def add_type(self, uritype):
+        super(StoredEntity, self).add_type(uritype)
+        self.context['addtype'].append(uritype)
+        
+    @_init_obj_wrapper
+    def del_type(self, uritype):
+        super(StoredEntity, self).del_type(uritype)
+        self.context['deltype'].append(uritype)
 
     @_init_obj_wrapper
     def list_parts(self):
@@ -78,12 +114,13 @@ class StoredEntity(Entity):
             pass
         self.manifest.add_triple("%s/%s" % (self.uri, "ROOT"), "dc:format", "application/rdf+xml")
         self.manifest.add_triple("%s/%s" % (self.uri, "MANIFEST"), "dc:format", "application/rdf+xml")
+        self.context = defaultdict(list)
 
     @_init_obj_wrapper
     def store_root(self):
         # only store the root if it has changed:
         if self.root.altered:
-            self.obj.putRoot(self.root)
+            return self.obj.putRoot(self.root)
 
     @_init_obj_wrapper
     def load_root(self, create_if_nonexistent=True):
@@ -91,16 +128,20 @@ class StoredEntity(Entity):
 
     @_init_obj_wrapper
     def put_stream(self, part_id, bytestream, mimetype=None, commit_metadata_changes=False):
-        self.obj.add_part(part_id, bytestream)
+        response = self.obj.add_part(part_id, bytestream)
         part_uri = self.uh.parse_uri("%s/%s" % (self.uri, part_id))
         self.add_triple(self.uri, "ore:aggregates", part_uri)
         self.add_triple(part_uri, "dcterms:isPartOf", self.uri)
         if mimetype:
             self.del_triple(part_uri, "dc:format", None)
             self.add_triple(part_uri, "dc:format", mimetype)
+        if response.get('checksum',None):
+            self.del_triple(part_uri, "ov:hasChecksum", None)
+            self.add_triple(part_uri, "ov:hasChecksum", "%s:%s" % response['checksum'])
             
         if commit_metadata_changes:
             self.commit()
+        return response
 
     @_init_obj_wrapper
     def get_stream_metadata(self, part_id):
@@ -146,7 +187,7 @@ class StoredEntity(Entity):
     def store_manifest(self):
         # only store the manifest if it has changed:
         if self.manifest.altered:
-            self.obj.putManifest(self.manifest)
+            return self.obj.putManifest(self.manifest)
 
     @_init_obj_wrapper
     def load_manifest(self, create_if_nonexistent=True):
