@@ -23,12 +23,13 @@ class FileMultiEntityFactory(object):
             raise NotAValidStoreName()
 
     def __init__(self, storage_dir='data', queue=None,
-                        prefix=SPECIAL_FILE_PREFIX, shorty_length=2):
+                        prefix=SPECIAL_FILE_PREFIX, shorty_length=2, hashing_type='md5'):
         self.stores = {}
         self.queue = queue
         self.storage_dir = storage_dir
         self.prefix = prefix
         self.shorty_length = shorty_length
+        self.hashing_type = hashing_type
         self.status = self.init_stores()
 
     def __getitem__(self, index):
@@ -69,7 +70,8 @@ class FileMultiEntityFactory(object):
                                                   storage_dir=store_path,
                                                   queue=self.queue,
                                                   prefix=self.prefix,
-                                                  shorty_length=self.shorty_length)
+                                                  shorty_length=self.shorty_length,
+                                                  hashing_type=self.hashing_type)
                         status[path] = True
                         self.stores[path] = store
                     except NotAPairtreeStoreException,e :
@@ -149,13 +151,16 @@ class FileMultiEntityFactory(object):
 
 class FileEntityFactory(object):
     def __init__(self, uri_base=URI_BASE, storage_dir=STORAGE_DIR,
-                        queue=None, prefix=SPECIAL_FILE_PREFIX, shorty_length=2, **context):
+                        queue=None, prefix=SPECIAL_FILE_PREFIX, shorty_length=2, 
+                        hashing_type='md5', **context):
         """uri_base must end in an /, : or an #. If neither is present a / will be appended."""
         if uri_base and not(uri_base.endswith('/') or uri_base.endswith('#') or uri_base.endswith(':')):
             uri_base = "%s/" % uri_base
         factory_f = FileStorageFactory()
+        self.hashing_type = hashing_type
         self._q = queue
-        self._s = factory_f.get_store(uri_base, storage_dir, prefix, shorty_length, queue, **context)
+        self._s = factory_f.get_store(uri_base, storage_dir, prefix, shorty_length, 
+                                      queue, hashing_type=hashing_type, **context)
         self._c = context
         # The store might have a different idea on what the prefix is:
         self.uri_base = self._s.uri_base
