@@ -60,8 +60,8 @@ SPECIAL_FILE_PREFIX = "_"
 
 class FileStorageFactory(object):
     def get_store(self, uri_base=URI_BASE, store_dir=STORAGE_DIR, prefix=SPECIAL_FILE_PREFIX,
-    shorty_length=2, queue=None, **context):
-        return FileStorageClient(uri_base, store_dir, prefix, shorty_length, queue, **context)
+    shorty_length=2, queue=None, hashing_type=None, **context):
+        return FileStorageClient(uri_base, store_dir, prefix, shorty_length, queue, hashing_type, **context)
 
 class FileStorageObject(object):
     def __init__(self, id, fs_store_client):
@@ -75,10 +75,10 @@ class FileStorageObject(object):
         """Store the root for this object - rdf can either be
         an RDFXML string or an RDFobject"""
         if isinstance(rdf, RDFobject):
-            self.fs._store_rdfobject(self.id, u'ROOT', rdf)
+            return self.fs._store_rdfobject(self.id, u'ROOT', rdf)
         else:
             r = RDFobject(self.id).from_string(rdf)
-            self.fs._store_rdfobject(self.id, u'ROOT', rdf)
+            return self.fs._store_rdfobject(self.id, u'ROOT', rdf)
 
     def getRoot(self):
         """RDFObject for this object"""
@@ -118,10 +118,10 @@ class FileStorageObject(object):
         """Store the manifest for this object - rdf can either be
         an RDFXML string or a Manifest"""
         if isinstance(rdf, Manifest):
-            self.fs._store_manifest(self.id, u'MANIFEST', rdf)
+            return self.fs._store_manifest(self.id, u'MANIFEST', rdf)
         else:
             r = Manifest().from_string(rdf)
-            self.fs._store_manifest(self.id, u'MANIFEST', rdf)
+            return self.fs._store_manifest(self.id, u'MANIFEST', rdf)
 
     def getManifest(self):
         """Manifest for this object"""
@@ -129,7 +129,7 @@ class FileStorageObject(object):
 
 
 class FileStorageClient(object):
-    def __init__(self, uri_base, store_dir, prefix, shorty_length,queue=None, **context):
+    def __init__(self, uri_base, store_dir, prefix, shorty_length,queue=None, hashing_type=None, **context):
         self.store_dir = store_dir
         self.uri_base = None
         if uri_base:
@@ -140,7 +140,12 @@ class FileStorageClient(object):
         self.shorty_length = shorty_length
         self.queue = queue
         self.context = context
-        self.storeclient = PairtreeStorageClient(uri_base, store_dir, shorty_length)
+        if hashing_type:
+            self.hashing_type = hashing_type
+            self.storeclient = PairtreeStorageClient(uri_base, store_dir, shorty_length,
+                                                     hashing_type=self.hashing_type)
+        else:
+            self.storeclient = PairtreeStorageClient(uri_base, store_dir, shorty_length)
         if self.storeclient.uri_base:
             self.uri_base = Namespace(self.storeclient.uri_base)
 
@@ -192,13 +197,13 @@ class FileStorageClient(object):
         if not version:
             version = self._get_latest_part(id, part_id) + 1
         part_name = "%s%s%s" % (part_id, self.prefix, version)
-        self.storeclient.put_stream(id, part_id, part_name, bytestream, buffer_size)
+        hexhash = self.storeclient.put_stream(id, part_id, part_name, bytestream, buffer_size)
         if self.queue != None:
             if version == 1:
-                self._log(id, 'c', 'Creating a part', part_id=part_id, version=version)
+                self._log(id, 'c', 'Creating a part', part_id=part_id, version=version, checksum=hexhash)
             else:
-                self._log(id, 'w', 'Updating a part', part_id=part_id, version=version)
-        return version
+                self._log(id, 'w', 'Updating a part', part_id=part_id, version=version, checksum=hexhash)
+        return {'version':version, 'checksum':hexhash}
 
     def _get_part(self, id, part_id, stream, version = False):
         if not self.storeclient.exists(id):
@@ -231,13 +236,13 @@ class FileStorageClient(object):
         bytestream = manifest.to_string()
         if isinstance(bytestream, unicode):
             bytestream = bytestream.encode('utf-8')
-        self.storeclient.put_stream(id, part_id, part_name, bytestream)
+        hexhash = self.storeclient.put_stream(id, part_id, part_name, bytestream)
         if self.queue != None:
             if version == 1:
-                self._log(id, 'c', 'Creating an RDF Manifest', part_id=part_id, version=version)
+                self._log(id, 'c', 'Creating an RDF Manifest', part_id=part_id, version=version, checksum=hexhash)
             else:
-                self._log(id, 'w', 'Updating an RDF Manifest', part_id=part_id, version=version)
-        return version
+                self._log(id, 'w', 'Updating an RDF Manifest', part_id=part_id, version=version, checksum=hexhash)
+        return {'version':version, 'checksum':hexhash}
 
     def _store_rdfobject(self, id, part_id, rdfobject, version=False):
         if not self.storeclient.exists(id):
@@ -248,13 +253,13 @@ class FileStorageClient(object):
         bytestream = rdfobject.to_string()
         if isinstance(bytestream, unicode):
             bytestream = bytestream.encode('utf-8')
-        self.storeclient.put_stream(id, part_id, part_name, bytestream)
+        hexhash = self.storeclient.put_stream(id, part_id, part_name, bytestream)
         if self.queue != None:
             if version == 1:
-                self._log(id, 'c', 'Creating an RDF Root', part_id=part_id, version=version)
+                self._log(id, 'c', 'Creating an RDF Root', part_id=part_id, version=version, checksum=hexhash)
             else:
-                self._log(id, 'w', 'Updating an RDF Root', part_id=part_id, version=version)
-        return version
+                self._log(id, 'w', 'Updating an RDF Root', part_id=part_id, version=version, checksum=hexhash)
+        return {'version':version, 'checksum':hexhash}
 
 
     def _get_rdfobject(self, id, part_id, version = False):
@@ -304,6 +309,10 @@ class FileStorageClient(object):
             if self.queue != None:
                 self._log(id, 'd', 'Deleting an object')
 
+    def log_audit(self, id, logcontext, context):
+        if self.queue != None:
+            self._log(id, 'metadatadelta', 'Metadata changes', _logcontext=logcontext, **context)
+
     def _log(self, id, action, label, **kw):
         msg = {}
         msg.update(kw)
@@ -312,7 +321,10 @@ class FileStorageClient(object):
         msg['action'] = action
         msg['label'] = label
         msg['uri_base'] = self.uri_base
+        # Get an ISO datetime for this
+        msg['timestamp'] = datetime.now().isoformat()
+
         try:
             self.queue.put(simplejson.dumps(msg))
         except Exception, e:
-            print "Logging failed"
+            raise e
